"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var React = _interopRequireWildcard(require("react"));

var _DatePickerItem = _interopRequireDefault(require("./DatePickerItem"));

var _pureRender = _interopRequireDefault(require("./utils/pureRender"));

var _time = require("./utils/time");

var _dataSource = require("./utils/dataSource");

var _prefix = require("./utils/prefix");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Uppercase first letter
 * @param {String} String
 */
const capitalize = ([first, ...rest]) => first.toUpperCase() + rest.join("");
/**
 * Judgment array
 * @param {any} val
 */


const isArray = val => Object.prototype.toString.apply(val) === "[object Array]";

class DatePicker extends React.Component {
  constructor(props) {
    super(props);

    _defineProperty(this, "state", {});

    this.state = {
      value: (0, _time.nextDate)(this.props.value)
    };
    this.handleFinishBtnClick = this.handleFinishBtnClick.bind(this);
    this.handleDateSelect = this.handleDateSelect.bind(this);
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    // update value of state
    const date = (0, _time.nextDate)(nextProps.value);

    if (date.getTime() !== this.state.value.getTime()) {
      this.setState({
        value: date
      });
    }
  }
  /**
   * When you swipe two datepickeritems at the same time.
   * Prevent dates from going out.
   */


  componentDidUpdate() {
    const value = this.state.value;
    const {
      min,
      max
    } = this.props;

    if (value.getTime() > max.getTime()) {
      this.setState({
        value: max
      });
    }

    if (value.getTime() < min.getTime()) {
      this.setState({
        value: min
      });
    }
  }
  /**
   * Optimization component, Prevents unnecessary rendering
   * Only props or state change or value before re-rendering
   *
   * @param  {Object} nextProps next props
   * @param  {Object} nextState next state
   * @return {Boolean}          Whether re-rendering
   */


  shouldComponentUpdate(nextProps, nextState) {
    const date = (0, _time.nextDate)(nextState.value);
    return date.getTime() !== this.state.value.getTime() || _pureRender.default.shouldComponentUpdate(nextProps, nextState, this.props, this.state);
  }
  /**
   * Click the Finish button event
   * @return {undefined}
   */


  handleFinishBtnClick() {
    this.props.onSelect(this.state.value);
  }
  /**
   * Select next date
   * @return {undefined}
   */


  handleDateSelect(value) {
    this.setState({
      value
    }, () => {
      this.props.onChange(value);
    });
  }
  /**
   * Format dateConfig
   * @param {*} dataConfig dateConfig attribute
   */


  normalizeDateConfig(dataConfig) {
    const configList = [];

    if (isArray(dataConfig)) {
      for (let i = 0; i < dataConfig.length; i++) {
        const value = dataConfig[i];

        if (typeof value === "string") {
          const lowerCaseKey = value.toLocaleLowerCase();
          configList.push({ ..._dataSource.dateConfigMap[lowerCaseKey],
            type: capitalize(lowerCaseKey)
          });
        }
      }
    } else {
      for (const key in dataConfig) {
        if (Object.prototype.hasOwnProperty.call(dataConfig, key)) {
          const lowerCaseKey = key.toLocaleLowerCase();

          if (Object.prototype.hasOwnProperty.call(_dataSource.dateConfigMap, lowerCaseKey)) {
            configList.push({ ..._dataSource.dateConfigMap[lowerCaseKey],
              ...dataConfig[key],
              type: capitalize(lowerCaseKey)
            });
          }
        }
      }
    }

    return configList;
  }

  checkMode(modeName) {
    if (!modeName || modeName === "") return _dataSource.mode.date;
    return _dataSource.mode[modeName];
  }

  checkHeaderFormat(modeName) {
    if (!modeName || modeName === "") {
      return _dataSource.headerFormatWithMode.date;
    }

    return _dataSource.headerFormatWithMode[modeName];
  }
  /**
   * render function
   * @return {Object} JSX object
   */


  render() {
    const {
      min,
      max,
      theme,
      dateConfig,
      mode,
      confirmText,
      cancelText,
      headerFormat,
      showHeader,
      showFooter,
      customHeader,
      showCaption
    } = this.props;
    const value = this.state.value;
    const themeClassName = ["default", "dark", "ios", "android", "android-dark"].indexOf(theme) === -1 ? "default" : theme;
    let dataConfigList = null; //handle config data with mode or custom

    if (!(0, _prefix.isEmpty)(dateConfig)) {
      dataConfigList = this.normalizeDateConfig(dateConfig);
    } else {
      dataConfigList = this.normalizeDateConfig(this.checkMode(mode));
    } // handle header with current mode


    let _headerFormat = "";

    if (!headerFormat && headerFormat === "") {
      _headerFormat = this.checkHeaderFormat(mode);
    }

    return /*#__PURE__*/React.createElement("div", {
      className: "datepicker ".concat(themeClassName)
    }, showHeader && /*#__PURE__*/React.createElement("div", {
      className: "datepicker-header"
    }, customHeader || (0, _time.convertDate)(value, _headerFormat)), showCaption && /*#__PURE__*/React.createElement("div", {
      className: "datepicker-caption"
    }, dataConfigList.map((item, index) => /*#__PURE__*/React.createElement("div", {
      key: index,
      className: "datepicker-caption-item"
    }, item.caption))), /*#__PURE__*/React.createElement("div", {
      className: "datepicker-content"
    }, dataConfigList.map((item, index) => /*#__PURE__*/React.createElement(_DatePickerItem.default, {
      key: index,
      value: value,
      min: min,
      max: max,
      step: item.step,
      type: item.type,
      format: item.format,
      onSelect: this.handleDateSelect
    }))), showFooter && /*#__PURE__*/React.createElement("div", {
      className: "datepicker-navbar"
    }, /*#__PURE__*/React.createElement("a", {
      className: "datepicker-navbar-btn",
      onClick: this.props.onCancel
    }, cancelText), /*#__PURE__*/React.createElement("a", {
      className: "datepicker-navbar-btn",
      onClick: this.handleFinishBtnClick
    }, confirmText)));
  }

}

var _default = DatePicker;
exports.default = _default;
//# sourceMappingURL=DatePicker.js.map