import * as React from "react";
import styled from "styled-components";
import useScrollbarSize from "react-scrollbar-size";
import {
    AppbarAction,
    AppbarBackAction,
    AppbarContent,
    AppbarHeader,
} from "react-native-simple-elements/components/Appbar";
import AuthorizedUserMenu from "../AuthorizedUserMenu";
import AccountIcon from "@mdi/svg/svg/account.svg";

const HeaderContainer = styled.div({
    zIndex: 3,
});

type Props = {
    title?: string,
    backButtonIcon?: React.ReactElement,
    onBackButtonClick?: () => void,
    loggedInUser?: Record<string, any>;
    isUserIconCircle?: boolean,
    shouldCheckScrollbarSize?: boolean,
    onLoginClick?: () => void,
    onViewProfileClick?: () => void,
    onLogoutClick?: () => void,
    children?: React.ReactNode;
};

const defaultProps = {
    isUserIconCircle: true,
    shouldCheckScrollbarSize: false,
};

const SimpleHeader = (props: Props) => {

    const {
        title,
        backButtonIcon,
        onBackButtonClick,
        children,
        loggedInUser,
        isUserIconCircle,
        shouldCheckScrollbarSize,
        onLoginClick,
        onViewProfileClick,
        onLogoutClick,
    } = props;

    const { width: scrollbarSize } = useScrollbarSize();

    const _handleBackActionPress = () => {
        if (typeof onBackButtonClick === "function") {
            onBackButtonClick();
        }
    };

    const _handleLoginClick = () => {
        if (typeof onLoginClick === "function") {
            onLoginClick();
        }
    };

    return (
        <HeaderContainer
            style={{
                marginRight: shouldCheckScrollbarSize ? scrollbarSize || 0 : 0,
            }}
        >
            <AppbarHeader>
                <AppbarBackAction
                    icon={backButtonIcon}
                    onPress={_handleBackActionPress}
                />
                <AppbarContent
                    title={title}
                />
                {children}
                {loggedInUser?.userId ?
                    <AuthorizedUserMenu
                        circle={isUserIconCircle}
                        loggedInUser={loggedInUser}
                        onViewProfileClick={onViewProfileClick}
                        onLogoutClick={onLogoutClick}
                    /> :
                    <AppbarAction
                        icon={AccountIcon}
                        onPress={_handleLoginClick}
                    />
                }
            </AppbarHeader>
        </HeaderContainer>
    );
};

SimpleHeader.defaultProps = defaultProps;

export default SimpleHeader;
